package org.jetbrains.kotlinx.statistics.binning

import kotlin.math.*

object BinStatUtil {
    private const val MAX_BIN_COUNT = 500

/*
    fun weightAtIndex(data: DataFrame): (Int) -> Double {
        if (data.has(TransformVar.WEIGHT)) {
            val weights = data.getNumeric(TransformVar.WEIGHT)
            return { index ->
                val weight = weights[index]
                SeriesUtil.asFinite(weight, 0.0)
            }
        }
        return { 1.0 }
    }

    // ToDo: need to deal fith n/a values (see DensityStat)
    fun weightVector(dataLength: Int, data: DataFrame): List<Double?> {
        return if (data.has(TransformVar.WEIGHT)) {
            data.getNumeric(TransformVar.WEIGHT)
        } else List(dataLength) { 1.0 }
    }*/

    fun binCountAndWidth(dataRange: Double, binsOption: BinsOption): CountAndWidth {
        val binCount: Int
        val binWidth: Double
        when(binsOption) {
            is ByNumber -> {
                binCount = binsOption.number
                binWidth = dataRange / binCount
            }
            is ByWidth -> {
                binWidth = binsOption.width
                var count = dataRange / binWidth
                count = min(MAX_BIN_COUNT.toDouble(), count)
                binCount = ceil(count).toInt()
            }
        }
        return CountAndWidth(binCount, binWidth)
    }

    fun <T> computeBinsByDouble(
        //data: DataFrame,
        //rangeX: DoubleSpan,
        values: List<T>,
        selector: (T) -> Double,
        //valuesToBin: List<Double>,
        binsRange: Pair<Double, Double>?,
        binsAlign: BinsAlign,
        binsOption: BinsOption
    ) = computeBinsByDouble(values, values.map(selector), binsRange, binsAlign, binsOption)



    fun <T> computeBinsByDouble(
        //data: DataFrame,
        //rangeX: DoubleSpan,
        values: List<T>,
        valuesToBin: List<Double>,
        binsRange: Pair<Double, Double>?,
        binsAlign: BinsAlign,
        binsOption: BinsOption
    ): BinModel<Double, List<T>> {
       // val valuesToBin = values.map(selector)
        val range = binsRange ?: valuesToBin.let {
            it.min() to it.max()
        }
        /*println(range)
        println(valuesX.min())*/
        var startX = range.first
        var spanX = range.second - startX

        // initial bin count/width
        var b: CountAndWidth = binCountAndWidth(spanX, binsOption)

        // adjusted bin count/width
        // extend the data range by `rangeExtendCoef` of binWidth on each ends (to allow limited horizontal adjustments)
       // val rangeExtendCoef = 0.7
       // startX -= b.width //* rangeExtendCoef
       // spanX += b.width //* 2 * rangeExtendCoef
       // b = binCountAndWidth(spanX, binsOption)
        val binCount = b.count
        val binWidth = b.width

        // optional horizontal adjustment (+/-0.5 bin width max)
        if (binsAlign !is None) {
            var minDelta = Double.MAX_VALUE
            val x = when(binsAlign) {
                is Boundary ->binsAlign.posValue
                is Center -> binsAlign.posValue
                None -> TODO()
            }

            for (i in 0 until binCount) {
                val binLeft = startX + i * binWidth
                val delta: Double
                if (binsAlign is Center) {
                    delta = x - (binLeft + binWidth / 2)
                } else {       // BOUNDARY
                    if (i == 0) {
                        minDelta = x - startX // init still
                    }
                    delta = x - (binLeft + binWidth)
                }

                if (abs(delta) < abs(minDelta)) {
                    minDelta = delta
                }
            }

            // max offset: +/-0.5 bin width
            val offset = minDelta % (binWidth / 2)
            startX += offset
        }


        return bins(
            values,
            valuesToBin,
            startX,
            binCount,
            binWidth,
            //weightAtIndex(data),
           // densityNormalizingFactor
        )

    }

   /* fun computeDotdensityStatSeries(
        rangeX: DoubleSpan,
        valuesX: List<Double?>,
        binsOption: binsOption
    ): BinsData {
        val spanX = rangeX.length
        val binWidth = if (spanX > 0.0) {
            binCountAndWidth(spanX, binsOption).width
        } else {
            // Only one stack of dots overall data
            1.0
        }

        return computeDotdensityBins(valuesX, binWidth)
    }
*/
    private fun<T> bins(
       values: List<T>,
       valuesToBin: List<Double>,
       startX: Double,
       binCount: Int,
       binWidth: Double,
       // weightAtIndex: (Int) -> Double,
        //densityNormalizingFactor: Double
    ): BinModel<Double, List<T>> {

        val binRanges = (0 until binCount).map {
            startX + binWidth * it to startX + binWidth * (it + 1)
        }
       val binValues = List(binCount) { mutableListOf<T>() }
        for (index in values.indices) {
            val binValue = valuesToBin[index]
            val value = values[index]
            val binIndex = floor((binValue - startX) / binWidth).toInt().let {
                if (it >= binCount) {
                    binCount - 1
                } else if (it < 0) {
                    0
                } else {
                    it
                }
            }

           /* println(startX)
            println(valuesX.min())
            println(value)*/
            binValues[binIndex].add(value)

//            if (!dataIndicesByBinIndex.containsKey(binIndex)) {
//                dataIndicesByBinIndex[binIndex] = ArrayList()
//            }

//            dataIndicesByBinIndex[binIndex]!!.add(dataIndex)
        }

       return BinModel(
           binRanges.zip(binValues).map {
               BinWithValue(Bin(it.first), it.second)
           }
       )

       /* val x = ArrayList<Double>()
        val counts = ArrayList<Double>()
        val densities = ArrayList<Double>()

        val x0 = startX + binWidth / 2
        for (i in 0 until binCount) {
            x.add(x0 + i * binWidth)

            var count = 0.0
            // some bins are left empty (not excluded from map)
            if (countByBinIndex.containsKey(i)) {
                count = countByBinIndex[i]!!.get()
            }

            counts.add(count)
            val density = count / totalCount * densityNormalizingFactor
            densities.add(density)
        }

//        return BinsData(x, counts, densities, dataIndicesByBinIndex)
        return BinsData(x, counts, densities, List(x.size) { binWidth })*/
    }

    /*private fun computeDotdensityBins(
        valuesX: List<Double?>,
        binWidth: Double
    ): BinsData {
        fun updateBinsData(
            binsData: BinsData,
            stack: MutableList<Double>,
            dataSize: Int
        ): BinsData {
            val v = (stack.last() - stack.first()) / 2.0

            return BinsData(
                binsData.x + listOf(stack.first() + v),
                binsData.count + listOf(stack.size.toDouble()),
                binsData.density + listOf(stack.size.toDouble() / dataSize),
                binsData.binWidth + listOf(binWidth)
            )
        }

        val sortedX = valuesX.filter { it?.isFinite() ?: false }
            .map { it!! }
            .sorted()
        var binsData = BinsData(emptyList(), emptyList(), emptyList(), emptyList())
        if (sortedX.isEmpty()) {
            return binsData
        }
        var stack = mutableListOf(sortedX.first())
        for (i in 1 until sortedX.size) {
            if (sortedX[i] - stack.first() < binWidth) {
                stack.add(sortedX[i])
                continue
            }
            binsData = updateBinsData(binsData, stack, sortedX.size)
            stack = mutableListOf(sortedX[i])
        }
        binsData = updateBinsData(binsData, stack, sortedX.size)

        return binsData
    }*/
/*
    class binsOption(
        binCount: Int, val binWidth: Double?  // optional
    ) {
        val binCount: Int = min(MAX_BIN_COUNT, max(1, binCount))

        fun hasBinWidth(): Boolean {
            return binWidth != null && binWidth > 0
        }
    }*/

    data class CountAndWidth(val count: Int, val width: Double)

}