/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jetbrains.kotlinx.statistics.math3.stat.descriptive

/**
 * Reporting interface for basic univariate statistics.
 *
 */
interface StatisticalSummary {
    /**
     * Returns the [
     * arithmetic mean ](http://www.xycoon.com/arithmetic_mean.htm) of the available values
     * @return The mean or Double.NaN if no values have been added.
     */
    val mean: Double

    /**
     * Returns the variance of the available values.
     * @return The variance, Double.NaN if no values have been added
     * or 0.0 for a single value set.
     */
    val variance: Double

    /**
     * Returns the standard deviation of the available values.
     * @return The standard deviation, Double.NaN if no values have been added
     * or 0.0 for a single value set.
     */
    val standardDeviation: Double

    /**
     * Returns the maximum of the available values
     * @return The max or Double.NaN if no values have been added.
     */
    val max: Double

    /**
     * Returns the minimum of the available values
     * @return The min or Double.NaN if no values have been added.
     */
    val min: Double

    /**
     * Returns the number of available values
     * @return The number of available values
     */
    val n: Long

    /**
     * Returns the sum of the values that have been added to Univariate.
     * @return The sum or Double.NaN if no values have been added
     */
    val sum: Double
}
