package org.jetbrains.kotlinx.statistics.dataframe.binning

import org.jetbrains.kotlinx.dataframe.ColumnSelector
import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.statistics.binning.Bin
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinStatUtil
import org.jetbrains.kotlinx.statistics.binning.BinsOption

class BinFrame<G>(val groupBy: GroupBy<Bin<Double>, G>) :
    GroupBy<Bin<Double>, G> by groupBy {
    //TODO()
}

fun <T> DataFrame<T>.binBy(
    binsOption: BinsOption = BinsOption.byNumber(15),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    binsRange: Pair<Number, Number>? = null,
    selector: ColumnSelector<T, Number>
): BinFrame<T> {
    val column = getColumn(selector).toList().map { it.toDouble() }
    val model = BinStatUtil.computeBinsByDouble(rows().toList(), column, binsRange?.let {
        it.first.toDouble() to it.second.toDouble()
    }, binsAlign, binsOption)
    return BinFrame(model.bins.let {
        val binColumn = DataColumn.create("bin", it.map { it.bin })
        val values = DataColumn.createFrameColumn("values", it.map { it.value.toDataFrame() })
        /*dataFrameOf("bin" to it.map { it.bin }, "values" to it.map {
            if (it.value.isNotEmpty()) {
                it.value.toDataFrame()
            } else DataFrame.Empty }).also {
                println(it.schema()) }*/
        dataFrameOf(binColumn, values)
    }.asGroupBy("values") as GroupBy<Bin<Double>, T>)
}
