package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.GroupBy
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.GroupedDataScope
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.kandy.layers.context.BoxplotLayerGroupedBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalGroupBy
import org.jetbrains.kotlinx.statistics.plotting.boxplot.models.StatBoxplotGroupBy
import org.jetbrains.kotlinx.statistics.plotting.boxplot.statBoxplot
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.StatBoxplotOutliersGroupBy
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.statBoxplotOutliers

@PublishedApi
internal fun<T, X> boxplotGrouped(
    parent: LayerCreatorScope,
    statBoxplotGroupBy: StatBoxplotGroupBy<T, X>,
    statBoxplotOutliersGroupBy: StatBoxplotOutliersGroupBy<T, X>,
    xAxisName: String,
    yAxisName: String,
    block: BoxplotLayerGroupedBuilder<T, X>.() -> Unit
) {
    BoxplotLayerGroupedBuilder<T, X>(parent, statBoxplotGroupBy, statBoxplotOutliersGroupBy).apply {
        boxes {
            x(Stat.x)
            yMin(Stat.min)
            lower(Stat.lower)
            middle(Stat.middle)
            upper(Stat.upper)
            yMax(Stat.max)

            key.columnNames().singleOrNull()?.let {
                fillColor(key[it])
            }

            x.axis.name = xAxisName
            y.axis.name = yAxisName
            position = Position.dodge(1.0)
        }

        outliers {
            x(Stat.x)
            y(Stat.y)
            position = Position.dodge(1.0)
        }
    }.apply(block).apply {
        parent.createLayer(boxes, {})
        if (outliers.show) {
            parent.createLayer(outliers, {})
        }
    }
}

@PublishedApi
internal fun<T, X> boxAndOutliers(
    groupBy: GroupBy<T, *>,
    x: DataColumn<X>,
    y: DataColumn<Number>,
    whiskerIQRRatio: Double,
): Pair<StatBoxplotGroupBy<T, X>, StatBoxplotOutliersGroupBy<T, X>> {
    return groupBy.statBoxplot(x, y, whiskerIQRRatio) to groupBy.statBoxplotOutliers(x, y, whiskerIQRRatio)
}
@PublishedApi
internal fun<T, X> boxAndOutliers(
    groupBy: GroupBy<T, *>,
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Number>,
    whiskerIQRRatio: Double,
): Pair<StatBoxplotGroupBy<T, X>, StatBoxplotOutliersGroupBy<T, X>> {
    return groupBy.statBoxplot(x, y, whiskerIQRRatio) to groupBy.statBoxplotOutliers(x, y, whiskerIQRRatio)
}

public fun<T, X> GroupedDataScope<T, *>.boxplot(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: BoxplotLayerGroupedBuilder<T, X>.() -> Unit = {}
) {
    val (boxes, outliers) = boxAndOutliers(internalGroupBy,x, y, whiskerIQRRatio)
    boxplotGrouped(this as LayerCreatorScope, boxes, outliers, x.name(), y.name(), block)
}

public fun <T> GroupedDataScope<T, *>.boxplot(
    x: String,
    y: String,
    whiskerIQRRatio: Double = 1.5,
    block: BoxplotLayerGroupedBuilder<T, Any?>.() -> Unit = {}
) {
    boxplot(column(x), column(y), whiskerIQRRatio, block)
}

public fun<T, X> GroupedDataScope<T, *>.boxplot(
    x: DataColumn<X>,
    y: DataColumn<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: BoxplotLayerGroupedBuilder<T, X>.() -> Unit = {}
) {
    val (boxes, outliers) = boxAndOutliers(internalGroupBy, x, y, whiskerIQRRatio)
    boxplotGrouped(this as LayerCreatorScope, boxes, outliers, x.name(), y.name(), block)
}
