package org.jetbrains.kotlinx.statistics.kandy.layers.context

import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.kandy.letsplot.layers.builders.BoxesBuilder
import org.jetbrains.kotlinx.kandy.letsplot.layers.builders.PointsBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.addDataset
import org.jetbrains.kotlinx.statistics.plotting.boxplot.models.StatBoxplotFrame
import org.jetbrains.kotlinx.statistics.plotting.boxplot.models.WithStatBoxplot
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.StatBoxplotOutliersFrame
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.WithStatBoxplotOutliers

class BoxplotLayerBuilder<X>(
    parent: LayerCreatorScope,
    statBoxplotFrame: StatBoxplotFrame<X>,
    statBoxplotOutliersFrame: StatBoxplotOutliersFrame<X>
) {

    @Suppress("INVISIBLE_MEMBER")
    class BoxplotBoxesBuilder<T>(parent: LayerCreatorScope, boxesFrame: StatBoxplotFrame<T>) :
        BoxesBuilder(parent, parent.addDataset(boxesFrame)),
        WithStatBoxplot<T> by boxesFrame

    @Suppress("INVISIBLE_MEMBER")
    class BoxplotOutliersBuilder<T>(parent: LayerCreatorScope, outliersFrame: StatBoxplotOutliersFrame<T>) :
        PointsBuilder(parent, parent.addDataset(outliersFrame)), WithStatBoxplotOutliers<T> by outliersFrame {
        var show: Boolean = true
    }

    val boxes = BoxplotBoxesBuilder(parent, statBoxplotFrame)
    val outliers = BoxplotOutliersBuilder(parent, statBoxplotOutliersFrame)

    public inline fun boxes(block: BoxplotBoxesBuilder<X>.() -> Unit) {
        boxes.apply(block)
    }

    public inline fun outliers(block: BoxplotOutliersBuilder<X>.() -> Unit) {
        outliers.apply(block)
    }
}
