package org.jetbrains.kotlinx.statistics.kandy.layers.context

import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.kandy.letsplot.layers.builders.BoxesBuilder
import org.jetbrains.kotlinx.kandy.letsplot.layers.builders.PointsBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.addDataset
import org.jetbrains.kotlinx.statistics.plotting.boxplot.models.StatBoxplotGroupBy
import org.jetbrains.kotlinx.statistics.plotting.boxplot.models.WithStatBoxplotGrouped
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.StatBoxplotOutliersGroupBy
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.WithStatBoxplotOutliersGrouped

class BoxplotLayerGroupedBuilder<T, X> internal constructor(
    parent: LayerCreatorScope,
    statBoxplotFrame: StatBoxplotGroupBy<T, X>,
    statBoxplotOutliersFrame: StatBoxplotOutliersGroupBy<T, X>
) {

    @Suppress("INVISIBLE_MEMBER")
    class BoxplotBoxesBuilder<T, X> internal constructor(
        parent: LayerCreatorScope,
        groupBy: StatBoxplotGroupBy<T, X>
    ) : BoxesBuilder(parent, parent.addDataset(groupBy)), WithStatBoxplotGrouped<T, X> by groupBy

    @Suppress("INVISIBLE_MEMBER")
    class BoxplotOutliersBuilder<T, X> internal constructor(
        parent: LayerCreatorScope,
        groupBy: StatBoxplotOutliersGroupBy<T, X>
    ) : PointsBuilder(parent, parent.addDataset(groupBy)), WithStatBoxplotOutliersGrouped<T, X> by groupBy {
        public var show: Boolean = true
    }

    val boxes = BoxplotBoxesBuilder(parent, statBoxplotFrame)
    val outliers = BoxplotOutliersBuilder(parent, statBoxplotOutliersFrame)

    public inline fun boxes(block: BoxplotBoxesBuilder<T, X>.() -> Unit) {
        boxes.apply(block)
    }

    public inline fun outliers(block: BoxplotOutliersBuilder<T, X>.() -> Unit) {
        outliers.apply(block)
    }
}
