package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.layers.context.CountPlotLayerBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.count.models.StatCountFrame
import org.jetbrains.kotlinx.statistics.plotting.count.statCount


@PublishedApi
internal inline fun<T> countPlot(parent: LayerCreatorScope, data: StatCountFrame<T>, axisName: String, weighted: Boolean, block: CountPlotLayerBuilder<T>.() -> Unit) {
    parent.createLayer(CountPlotLayerBuilder<T>(parent, data)) {
        x(Stat.x)
        if (weighted) {
            y(Stat.countWeighted)
        } else {
            y(Stat.count)
        }
        x.axis.name = axisName
        block()
    }
}

public inline fun<T> DataFramePlotBuilder<*>.countPlot(
    x: Iterable<T>,
    weights: Iterable<Number>? = null,
    block: CountPlotLayerBuilder<T>.() -> Unit = {}
) {
    countPlot(this, statCount(x, weights), "x", weights != null, block)
}

public inline fun<T> DataFramePlotBuilder<*>.countPlot(
    x: ColumnAccessor<T>,
    weights: ColumnAccessor<Number>? = null,
    block: CountPlotLayerBuilder<T>.() -> Unit = {}
) {
    countPlot(this, internalDataFrame.statCount(x, weights), x.name(), weights != null, block)
}

public inline fun DataFramePlotBuilder<*>.countPlot(
    x: String,
    weights: String? = null,
    block: CountPlotLayerBuilder<Any?>.() -> Unit = {}
) {
    countPlot(this, internalDataFrame.statCount(x, weights), x, weights != null, block)
}

public inline fun<T> DataFramePlotBuilder<*>.countPlot(
    x: DataColumn<T>,
    weights: DataColumn<Number>? = null,
    block: CountPlotLayerBuilder<T>.() -> Unit = {}
) {
    countPlot(this, statCount(x, weights), x.name(), weights != null, block)
}
