package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.GroupedDataScope
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.kandy.layers.context.CountPlotLayerGroupedBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalGroupBy
import org.jetbrains.kotlinx.statistics.plotting.count.models.StatCountGroupBy
import org.jetbrains.kotlinx.statistics.plotting.count.statCount


@PublishedApi
internal inline fun <T, X> countPlotGrouped(
    parent: LayerCreatorScope,
    data: StatCountGroupBy<T, X>,
    axisName: String,
    weighted: Boolean,
    block: CountPlotLayerGroupedBuilder<T, X>.() -> Unit
) {
    parent.createLayer(CountPlotLayerGroupedBuilder<T, X>(parent, data)) {
        x(Stat.x)
        if (weighted) {
            y(Stat.countWeighted)
        } else {
            y(Stat.count)
        }
        key.columnNames().singleOrNull()?.let {
            fillColor(key[it])
        }
        position = Position.dodge()
        x.axis.name = axisName
        block()
    }
}


public inline fun <T, X> GroupedDataScope<T, *>.countPlot(
    x: ColumnAccessor<X>,
    weights: ColumnAccessor<Number>? = null,
    block: CountPlotLayerGroupedBuilder<T, X>.() -> Unit = {}
) {
    countPlotGrouped<T, X>(this as LayerCreatorScope, internalGroupBy.statCount(x, weights), x.name(), weights != null, block)
}

public inline fun <T> GroupedDataScope<T, *>.countPlot(
    x: String,
    weights: String? = null,
    block: CountPlotLayerGroupedBuilder<T, Any?>.() -> Unit = {}
) {
    countPlotGrouped(this as LayerCreatorScope, internalGroupBy.statCount(x, weights), x, weights != null, block)
}

public inline fun <T, X> GroupedDataScope<T, *>.countPlot(
    x: DataColumn<X>,
    weights: DataColumn<Number>? = null,
    block: CountPlotLayerGroupedBuilder<T, X>.() -> Unit = {}
) {
    countPlotGrouped(this as LayerCreatorScope, internalGroupBy.statCount(x, weights), x.name(), weights != null, block)
}
