package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.layers.context.DensityPlotLayerBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.density.BandWidth
import org.jetbrains.kotlinx.statistics.plotting.density.Kernel
import org.jetbrains.kotlinx.statistics.plotting.density.models.StatDensityFrame
import org.jetbrains.kotlinx.statistics.plotting.density.statDensity

@Suppress("INVISIBLE_MEMBER")
@PublishedApi
internal fun densityPlot(parent: LayerCreatorScope, data: StatDensityFrame, axisName: String, weighted: Boolean, block: DensityPlotLayerBuilder.() -> Unit) {
    parent.createLayer(DensityPlotLayerBuilder(parent, data)) {
        x(Stat.x)
        if (weighted) {
            y(Stat.densityWeighted)
        } else {
            y(Stat.density)
        }
        x.axis.name = axisName
        block()
    }
}

public fun DataFramePlotBuilder<*>.densityPlot(
    x: Iterable<Number>,
    weights: Iterable<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: DensityPlotLayerBuilder.() -> Unit = {}
) {
    densityPlot(this, statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth), "x", weights != null, block)
}

public  fun DataFramePlotBuilder<*>.densityPlot(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: DensityPlotLayerBuilder.() -> Unit = {}
) {
    densityPlot(this, internalDataFrame.statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth), x.name(),weights != null, block)
}

public  fun DataFramePlotBuilder<*>.densityPlot(
    x: String,
    weights: String? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: DensityPlotLayerBuilder.() -> Unit = {}
) {
   densityPlot(column(x), weights?.let { column("it") }, n, trim, adjust, kernel, fullScanMax, bandWidth, block)
}

public  fun DataFramePlotBuilder<*>.densityPlot(
    x: DataColumn<Number>,
    weights: DataColumn<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: DensityPlotLayerBuilder.() -> Unit = {}
) {
    densityPlot(this, statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth), x.name(), weights != null, block)
}