package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.GroupBy
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.GroupedDataScope
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.layers.context.DensityPlotLayerGroupedBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalGroupBy
import org.jetbrains.kotlinx.statistics.plotting.density.BandWidth
import org.jetbrains.kotlinx.statistics.plotting.density.Kernel
import org.jetbrains.kotlinx.statistics.plotting.density.models.StatDensityGroupBy
import org.jetbrains.kotlinx.statistics.plotting.density.statDensity

@PublishedApi
internal inline fun <T> densityPlotGrouped(
    parent: LayerCreatorScope,
    data: StatDensityGroupBy<T>,
    axisName: String,
    weighted: Boolean,
    block: DensityPlotLayerGroupedBuilder<T>.() -> Unit
) {
    parent.createLayer(DensityPlotLayerGroupedBuilder<T>(parent, data)) {
        x(Stat.x)
        if (weighted) {
            y(Stat.densityWeighted)
        } else {
            y(Stat.density)
        }
        x.axis.name = axisName
        key.columnNames().singleOrNull()?.let {
            fillColor(key[it])
            borderLine.color(key[it])
        }
        //position = Position.dodge()
        block()
    }
}



public inline fun <T> GroupedDataScope<T, *>.densityPlot(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: DensityPlotLayerGroupedBuilder<T>.() -> Unit = {}
) {
    val groupBy = internalGroupBy as GroupBy<T, *>
    val data = groupBy.statDensity(x,weights, n, trim, adjust, kernel, fullScanMax, bandWidth)
    densityPlotGrouped(this as LayerCreatorScope, data, x.name(), weights != null, block)
}

public inline fun <T> GroupedDataScope<T, *>.densityPlot(
    x: String,
    weights: String? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: DensityPlotLayerGroupedBuilder<T>.() -> Unit = {}
) {
   this.densityPlot(column(x), weights?.let { column(it) }, n, trim, adjust, kernel, fullScanMax, bandWidth, block)
}


public inline fun <T> GroupedDataScope<T, *>.densityPlot(
    x: DataColumn<Number>,
    weights: DataColumn<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: DensityPlotLayerGroupedBuilder<T>.() -> Unit = {}
) {
    densityPlotGrouped(this as LayerCreatorScope, (internalGroupBy as GroupBy<T, *>).statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth), x.name(), weights != null, block)

}
