package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.layers.context.HeatmapBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.count2d.models.StatCount2DFrame
import org.jetbrains.kotlinx.statistics.plotting.count2d.statCount2D


@PublishedApi
internal inline fun <X, Y> heatmap(
    parent: LayerCreatorScope,
    data: StatCount2DFrame<X, Y>,
    xAxisName: String,
    yAxisName: String,
    weighted: Boolean,
    block: HeatmapBuilder<X, Y>.() -> Unit
) {
    parent.createLayer(HeatmapBuilder<X, Y>(parent, data)) {
        x(Stat.x)
        y(Stat.y)
        if (weighted) {
            fillColor(Stat.countWeighted)
        } else {
            fillColor(Stat.count)
        }
        x.axis.name = xAxisName
        y.axis.name = yAxisName
        block()
    }
}

public inline fun <reified X, reified Y> LayerCreatorScope.heatmap(
    x: Iterable<X>,
    y: Iterable<Y>,
    weights: Iterable<Number>? = null,
    block: HeatmapBuilder<X, Y>.() -> Unit = {}
) {
    heatmap(this, statCount2D(x, y, weights), "x", "y", weights != null, block)
}

public inline fun <X, Y> LayerCreatorScope.heatmap(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Y>,
    weights: ColumnAccessor<Number>? = null,
    block: HeatmapBuilder<X, Y>.() -> Unit = {}
) {
    heatmap(this, internalDataFrame.statCount2D(x, y, weights), x.name(), y.name(), weights != null, block)
}

public inline fun LayerCreatorScope.heatmap(
    x: String,
    y: String,
    weights: String? = null,
    block: HeatmapBuilder<Any?, Any?>.() -> Unit = {}
) {
    heatmap(this, internalDataFrame.statCount2D(x, y, weights), x, y,weights != null, block)
}

public inline fun <X, Y> LayerCreatorScope.heatmap(
    x: DataColumn<X>,
    y: DataColumn<Y>,
    weights: DataColumn<Number>? = null,
    block: HeatmapBuilder<X, Y>.() -> Unit = {}
) {
    heatmap(this, statCount2D(x, y, weights), x.name(), y.name(), weights != null, block)
}
