package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinsOption
import org.jetbrains.kotlinx.statistics.kandy.layers.context.HistogramBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.bin.models.StatBinFrame
import org.jetbrains.kotlinx.statistics.plotting.bin.statBin

@Suppress("INVISIBLE_MEMBER")
@PublishedApi
internal fun histogram(parent: LayerCreatorScope, data: StatBinFrame, axisName: String, weighted: Boolean, block: HistogramBuilder.() -> Unit) {
    parent.createLayer(HistogramBuilder(parent, data).apply {
        x(Stat.x)
        if (weighted) {
            y(Stat.countWeighted)
        } else {
            y(Stat.count)
        }
        x.axis.name = axisName
    }, block)
}

public fun DataFramePlotBuilder<*>.histogram(
    x: Iterable<Number>,
    weights: Iterable<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    block: HistogramBuilder.() -> Unit = {}
) {
    histogram(this, statBin(x, weights, binsOption, binsAlign), "x", weights != null, block)
}

public  fun DataFramePlotBuilder<*>.histogram(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    block: HistogramBuilder.() -> Unit = {}
) {
    histogram(this, internalDataFrame.statBin(x, weights, binsOption, binsAlign), x.name(),weights != null, block)
}

public  fun DataFramePlotBuilder<*>.histogram(
    x: String,
    weights: String? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    block: HistogramBuilder.() -> Unit = {}
) {
   histogram(column(x), weights?.let { column("it") }, binsOption, binsAlign, block)
}

public  fun DataFramePlotBuilder<*>.histogram(
    x: DataColumn<Number>,
    weights: DataColumn<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    block: HistogramBuilder.() -> Unit = {}
) {
    histogram(this, statBin(x, weights, binsOption, binsAlign), x.name(), weights != null, block)
}