package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.GroupedDataScope
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinsOption
import org.jetbrains.kotlinx.statistics.kandy.layers.context.HistogramGroupedBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalGroupBy
import org.jetbrains.kotlinx.statistics.plotting.bin.models.StatBinGroupBy
import org.jetbrains.kotlinx.statistics.plotting.bin.statBin

@PublishedApi
internal inline fun <T> histogramGrouped(
    parent: LayerCreatorScope,
    data: StatBinGroupBy<T>,
    axisName: String,
    weighted: Boolean,
    block: HistogramGroupedBuilder<T>.() -> Unit
) {
    parent.createLayer(HistogramGroupedBuilder<T>(parent, data)) {
        x(Stat.x)
        if (weighted) {
            y(Stat.countWeighted)
        } else {
            y(Stat.count)
        }
        x.axis.name = axisName
        key.columnNames().singleOrNull()?.let {
            fillColor(key[it])
        }
        position = Position.dodge()
        block()
    }
}



public inline fun <T> GroupedDataScope<T, *>.histogram(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    block: HistogramGroupedBuilder<T>.() -> Unit = {}
) {
    val data = internalGroupBy.statBin(x,weights, binsOption, binsAlign)
    histogramGrouped(this as LayerCreatorScope, data, x.name(), weights != null, block)
}

public inline fun <T> GroupedDataScope<T, *>.histogram(
    x: String,
    weights: String? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    block: HistogramGroupedBuilder<T>.() -> Unit = {}
) {
   this.histogram(column(x), weights?.let { column(it) }, binsOption, binsAlign, block)
}


public inline fun <T> GroupedDataScope<T, *>.histogram(
    x: DataColumn<Number>,
    weights: DataColumn<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    block: HistogramGroupedBuilder<T>.() -> Unit = {}
) {
    histogramGrouped(this as LayerCreatorScope, internalGroupBy.statBin(x, weights, binsOption, binsAlign), x.name(), weights != null, block)
}
