package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.layers.context.SmoothLineLayerContext
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.smooth.SmoothMethod
import org.jetbrains.kotlinx.statistics.plotting.smooth.models.StatSmoothFrame
import org.jetbrains.kotlinx.statistics.plotting.smooth.statSmooth

@Suppress("INVISIBLE_MEMBER")
@PublishedApi
internal fun smoothLine(parent: LayerCreatorScope, data: StatSmoothFrame, xAxisName: String, yAxisName: String, block: SmoothLineLayerContext.() -> Unit) {
    parent.createLayer(SmoothLineLayerContext(parent, data))  {
        @Suppress("INVISIBLE_MEMBER", "INVISIBLE_REFERENCE")
        println((datasetBuilder as DatasetBuilderImpl).buffer)
        x(Stat.x)
        y(Stat.y)
        x.axis.name = xAxisName
        y.axis.name = yAxisName
        block()
    }
}

public fun DataFramePlotBuilder<*>.smoothLine(
    x: Iterable<Number>,
    y: Iterable<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: SmoothLineLayerContext.() -> Unit = {}
) {
    smoothLine(this, statSmooth(x, y, method, smootherPointCount), "x", "y", block)
}

public  fun DataFramePlotBuilder<*>.smoothLine(
    x: ColumnAccessor<Number>,
    y: ColumnAccessor<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: SmoothLineLayerContext.() -> Unit = {}
) {
    smoothLine(this, internalDataFrame.statSmooth(x, y, method, smootherPointCount), x.name(), y.name(), block)
}

public  fun DataFramePlotBuilder<*>.smoothLine(
    x: String,
    y: String,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: SmoothLineLayerContext.() -> Unit = {}
) {
    smoothLine(this, internalDataFrame.statSmooth(x, y, method, smootherPointCount), x, y, block)
}

public  fun DataFramePlotBuilder<*>.smoothLine(
    x: DataColumn<Number>,
    y: DataColumn<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: SmoothLineLayerContext.() -> Unit = {}
) {
    smoothLine(this, statSmooth(x, y, method, smootherPointCount), x.name(),y.name(), block)
}
