package org.jetbrains.kotlinx.statistics.kandy.layers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.GroupBy
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.GroupedDataScope
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.layers.context.SmoothLineGroupedBuilder
import org.jetbrains.kotlinx.statistics.kandy.util.internalGroupBy
import org.jetbrains.kotlinx.statistics.plotting.smooth.SmoothMethod
import org.jetbrains.kotlinx.statistics.plotting.smooth.models.StatSmoothGroupBy
import org.jetbrains.kotlinx.statistics.plotting.smooth.statSmooth

@PublishedApi
internal inline fun <T> smoothLineGrouped(
    parent: LayerCreatorScope,
    data: StatSmoothGroupBy<T>,
    xAxisName: String,
    yAxisName: String,
    block: SmoothLineGroupedBuilder<T>.() -> Unit
) {
    parent.createLayer(SmoothLineGroupedBuilder<T>(parent, data)) {
        x(Stat.x)
        y(Stat.y)
        x.axis.name = xAxisName
        y.axis.name = yAxisName
        key.columnNames().singleOrNull()?.let {
            color(key[it])
        }
        block()
    }
}



public inline fun <T> GroupedDataScope<T, *>.smoothLine(
    x: ColumnAccessor<Number>,
    y: ColumnAccessor<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: SmoothLineGroupedBuilder<T>.() -> Unit = {}
) {
    val groupBy = internalGroupBy as GroupBy<T, *>
    val data = groupBy.statSmooth(x,y, method, smootherPointCount)
    smoothLineGrouped(this as LayerCreatorScope, data, x.name(), y.name(), block)
}

public inline fun <T> GroupedDataScope<T, *>.smoothLine(
    x: String,
    y: String,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: SmoothLineGroupedBuilder<T>.() -> Unit = {}
) {
    val groupBy = internalGroupBy as GroupBy<T, *>
    val data = groupBy.statSmooth(x,y, method, smootherPointCount)
    smoothLineGrouped(this as LayerCreatorScope, data, x, y, block)}


public inline fun <T> GroupedDataScope<T, *>.smoothLine(
    x: DataColumn<Number>,
    y: DataColumn<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: SmoothLineGroupedBuilder<T>.() -> Unit = {}
) {
    val groupBy = internalGroupBy as GroupBy<T, *>
    val data = groupBy.statSmooth(x,y, method, smootherPointCount)
    smoothLineGrouped(this as LayerCreatorScope, data, x.name(), y.name(), block)
}
