package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.BoxplotStatPlot
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.kandy.layers.boxAndOutliers
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.HistogramPlotContext
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.BoxplotStatPlotContext
import org.jetbrains.kotlinx.statistics.selectors.*



fun boxplot(
    y: Iterable<Number>,
    showOutliers: Boolean = true,
    whiskerIQRRatio: Double = 1.5,
    xAxisName: String = "",
    yAxisName: String = "",
): BoxplotStatPlot<String> {
    return BoxplotStatPlot(boxAndOutliers(y, whiskerIQRRatio, xAxisName), xAxisName, yAxisName, showOutliers)
}

fun <X> DataFrame<*>.boxplot(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Number>,
    showOutliers: Boolean = true,
    whiskerIQRRatio: Double = 1.5,
): BoxplotStatPlot<X> {
    return BoxplotStatPlot(boxAndOutliers(this, x, y, whiskerIQRRatio), x.name(), y.name(), showOutliers)
}

fun DataFrame<*>.boxplot(
    x: String,
    y: String,
    showOutliers: Boolean = true,
    whiskerIQRRatio: Double = 1.5,
): BoxplotStatPlot<Any?> {
    return boxplot(column(x), column(y), showOutliers, whiskerIQRRatio)
}


fun<T, X> DataFrame<T>.boxplot(
    showOutliers: Boolean = true,
    whiskerIQRRatio: Double = 1.5,
    selector: StatXYNumSelectorDSL<T, X>
): BoxplotStatPlot<X> {
    val appliedSelector = StatXYNumSelector<T, X>(this).apply(selector)
    return BoxplotStatPlot(boxAndOutliers(appliedSelector.x, appliedSelector.y, whiskerIQRRatio),
        appliedSelector.x.name(),
        appliedSelector.y.name(),
        showOutliers
        )
}

fun<X> BoxplotStatPlot<X>.configure(block: BoxplotStatPlotContext<X>.() -> Unit): Plot {
    return BoxplotStatPlotContext(this).apply {
        boxes {
            x(Stat.x)
            yMin(Stat.min)
            lower(Stat.lower)
            middle(Stat.middle)
            upper(Stat.upper)
            yMax(Stat.max)

            x.axis.name = xAxisName
            y.axis.name = yAxisName
        }

        outliers {
            show = showOutliers
            x(Stat.x)
            y(Stat.y)
        }
        block()
    }.toPlot()
}
