package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.GroupBy
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.api.concat
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.BoxplotStatGroupedPlot
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.BoxplotStatGroupedPlotContext
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.kandy.layers.boxAndOutliers

import org.jetbrains.kotlinx.statistics.selectors.*


fun <T, X> GroupBy<T, *>.boxplot(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Number>,
    showOutliers: Boolean = true,
    whiskerIQRRatio: Double = 1.5,
): BoxplotStatGroupedPlot<T, X> {
    return BoxplotStatGroupedPlot(boxAndOutliers(this, x, y, whiskerIQRRatio), x.name(), y.name(), showOutliers)
}

fun <T> GroupBy<T, *>.boxplot(
    x: String,
    y: String,
    showOutliers: Boolean = true,
    whiskerIQRRatio: Double = 1.5,
): BoxplotStatGroupedPlot<T, Any?> {
    return boxplot(column(x), column(y), showOutliers, whiskerIQRRatio)
}


fun<T, G, X> GroupBy<T, G>.boxplot(
    showOutliers: Boolean = true,
    whiskerIQRRatio: Double = 1.5,
    selector: StatXYNumSelectorDSL<G, X>
): BoxplotStatGroupedPlot<T, X> {
    val appliedSelector = StatXYNumSelector<G, X>(this.groups.concat()).apply(selector)
    return BoxplotStatGroupedPlot(boxAndOutliers(this, appliedSelector.x, appliedSelector.y, whiskerIQRRatio),
        appliedSelector.x.name(),
        appliedSelector.y.name(),
        showOutliers
        )
}

fun<T, X> BoxplotStatGroupedPlot<T, X>.configure(block: BoxplotStatGroupedPlotContext<T, X>.() -> Unit): Plot {
    return BoxplotStatGroupedPlotContext(this).apply {
        boxes {
            x(Stat.x)
            yMin(Stat.min)
            lower(Stat.lower)
            middle(Stat.middle)
            upper(Stat.upper)
            yMax(Stat.max)

            key.columnNames().singleOrNull()?.let {
                fillColor(key[it])
            }

            x.axis.name = xAxisName
            y.axis.name = yAxisName
            position = Position.dodge(1.0)
        }

        outliers {
            show = showOutliers
            x(Stat.x)
            y(Stat.y)
            position = Position.dodge(1.0)
        }
        block()
    }.toPlot()
}
