package org.jetbrains.kotlinx.statistics.kandy.statplots.context

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.kandy.dsl.internal.*
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.internal.*
import org.jetbrains.kotlinx.statistics.kandy.layers.context.*
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.BoxplotStatGroupedPlot
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*
import org.jetbrains.kotlinx.kandy.ir.data.TableData

@Suppress("INVISIBLE_MEMBER", "INVISIBLE_REFERENCE", "CANNOT_OVERRIDE_INVISIBLE_MEMBER")
public class BoxplotStatGroupedPlotContext<T, X>(
    boxplotStatGroupedPlot: BoxplotStatGroupedPlot<T, X>,
) : CustomPlotBuilder() {
    private val plotBuilder = object : MultiLayerPlotBuilder() {
        override val datasetBuilders: MutableList<DatasetBuilder> = mutableListOf()
        override fun addDataset(dataset: TableData, initialBuilder: DatasetBuilder?): Int {
            datasetBuilders.add(DatasetBuilderImpl(dataset,null))
            return datasetBuilders.lastIndex
        }
        override fun addEmptyDataset(): Int {
            return addDataset(NamedData(DataFrame.Empty), null)
        }
    }

    internal override val plotFeatures
        get () = plotBuilder.plotFeatures
    internal override val bindingHandler
        get() = plotBuilder.bindingHandler
    override val datasetBuilder
        get() = plotBuilder.datasetBuilder

    val boxes = BoxplotLayerGroupedBuilder.BoxplotBoxesBuilder<T, X>(
            plotBuilder, boxplotStatGroupedPlot.statBoxplotGroupBy
        )
    val outliers = BoxplotLayerGroupedBuilder.BoxplotOutliersBuilder<T, X>(
        plotBuilder, boxplotStatGroupedPlot.statBoxplotOutliersGroupBy
    )

    public inline fun boxes(block: BoxplotLayerGroupedBuilder.BoxplotBoxesBuilder<T, X>.() -> Unit) {
        boxes.apply(block)
    }

    public inline fun outliers(block: BoxplotLayerGroupedBuilder.BoxplotOutliersBuilder<T, X>.() -> Unit) {
        outliers.apply(block)
    }

    @Suppress("INVISIBLE_MEMBER")
    override fun toPlot(): Plot {
        plotBuilder.createLayer(boxes, {})
        if (outliers.show) {
            plotBuilder.createLayer(outliers, {})
        }
        return plotBuilder.toPlot()
    }

}
