package org.jetbrains.kotlinx.statistics.kandy.statplots.context

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.kandy.dsl.internal.*
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.NamedData
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.ir.data.TableData
import org.jetbrains.kotlinx.kandy.ir.feature.FeatureName
import org.jetbrains.kotlinx.kandy.ir.feature.PlotFeature
import org.jetbrains.kotlinx.statistics.kandy.layers.context.BoxplotLayerBuilder
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.BoxplotStatPlot

@Suppress("INVISIBLE_MEMBER", "CANNOT_OVERRIDE_INVISIBLE_MEMBER", "INVISIBLE_REFERENCE")
public class BoxplotStatPlotContext<X>(
    boxplotStatPlot: BoxplotStatPlot<X>
) : CustomPlotBuilder() {

    private val plotBuilder = object : MultiLayerPlotBuilder() {
        override val datasetBuilders: MutableList<DatasetBuilder> = mutableListOf()
        override fun addDataset(dataset: TableData, initialBuilder: DatasetBuilder?): Int {
            datasetBuilders.add(DatasetBuilderImpl(dataset, null))
            return datasetBuilders.lastIndex
        }
        override fun addEmptyDataset(): Int {
            return addDataset(NamedData(DataFrame.Empty), null)
        }
    }

    internal override val plotFeatures
        get () = plotBuilder.plotFeatures
    internal override val bindingHandler
        get() = plotBuilder.bindingHandler
    override val datasetBuilder
        get() = plotBuilder.datasetBuilder

    val boxes = BoxplotLayerBuilder.BoxplotBoxesBuilder<X>(plotBuilder, boxplotStatPlot.statBoxplotFrame)
    val outliers = BoxplotLayerBuilder.BoxplotOutliersBuilder<X>(plotBuilder, boxplotStatPlot.statBoxplotOutliersFrame)

    public inline fun boxes(block: BoxplotLayerBuilder.BoxplotBoxesBuilder<X>.() -> Unit) {
        boxes.apply(block)
    }

    public inline fun outliers(block: BoxplotLayerBuilder.BoxplotOutliersBuilder<X>.() -> Unit) {
        outliers.apply(block)
    }

    @Suppress("INVISIBLE_MEMBER")
    override fun toPlot(): Plot {
        plotBuilder.createLayer(boxes, {})
        if (outliers.show) {
            plotBuilder.createLayer(outliers, {})
        }
        return plotBuilder.toPlot()
    }

}
