package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelector
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.CountPlotContext
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.CountPlot
import org.jetbrains.kotlinx.statistics.plotting.count.statCount
import org.jetbrains.kotlinx.statistics.selectors.StatXSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXSelectorDSL


fun<X> countPlot(
    x: Iterable<X>,
    weights: Iterable<Number>? = null,
): CountPlot<X> {
    return CountPlot(statCount(x, weights),"x", weights != null)
}

fun<X> DataFrame<*>.countPlot(
    x: ColumnAccessor<X>,
    weights: ColumnAccessor<Number>? = null,
): CountPlot<X> {
    return CountPlot(statCount(x, weights), x.name(), weights != null)
}

fun DataFrame<*>.countPlot(
    xColumnName: String,
    weightsColumnName: String? = null,
): CountPlot<Any?> {
    return CountPlot(statCount(xColumnName, weightsColumnName), xColumnName, weightsColumnName != null)
}


fun<T, X> DataFrame<T>.countPlot(
    selector: StatXSelectorDSL<T, X>
): CountPlot<X> {
    val appliedSelector = StatXSelector<T, X>(this).apply(selector)
    return CountPlot(statCount(selector), appliedSelector.x.name(), appliedSelector.weights != null)
}

fun<X> CountPlot<X>.configure(block: CountPlotContext<X>.() -> Unit): Plot {
    return CountPlotContext(this).apply {
        x(Stat.x)
        if (weighted) {
            y(Stat.countWeighted)
        } else {
            y(Stat.count)
        }
        x.axis.name = name
        position = Position.dodge()
        block()
    }.toPlot()
}
