package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.api.GroupBy
import org.jetbrains.kotlinx.dataframe.api.concat
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinsOption
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.CountGroupedPlotContext
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.CountGroupedPlot
import org.jetbrains.kotlinx.statistics.plotting.count.statCount
import org.jetbrains.kotlinx.statistics.selectors.StatXSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXSelectorDSL


fun<T, G> GroupBy<T, G>.countPlot(
    x: String,
    weightsColumnName: String? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0)
): CountGroupedPlot<T, Any?> {
    return CountGroupedPlot(statCount(x, weightsColumnName), x, weightsColumnName!= null)
}

fun<T, G, X> GroupBy<T, G>.countPlot(
    x: ColumnAccessor<X>,
    weight: ColumnAccessor<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0)
): CountGroupedPlot<T, X> {
    return CountGroupedPlot(statCount(x, weight), x.name(), weight != null)
}


fun<T, G, X> GroupBy<T, G>.countPlot(
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    selector: StatXSelectorDSL<G, X>
): CountGroupedPlot<T, X> {
    val appliedSelector = StatXSelector<G, X>(this.groups.concat()).apply(selector)
    return CountGroupedPlot(statCount(selector), appliedSelector.x.name(), appliedSelector.weights != null)
}

fun<T, X> CountGroupedPlot<T, X>.configure(block: CountGroupedPlotContext<T, X>.() -> Unit): Plot {
    return CountGroupedPlotContext(this).apply {
        x(Stat.x)
        if (weighted) {
            y(Stat.countWeighted)
        } else {
            y(Stat.count)
        }
        key.columnNames().singleOrNull()?.let {
            fillColor(key[it])
        }
        position = Position.dodge()
        x.axis.name = name
        block()
    }.toPlot()
}
