package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinsOption
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelector
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.DensityPlotContext
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.DensityPlot
import org.jetbrains.kotlinx.statistics.plotting.density.BandWidth
import org.jetbrains.kotlinx.statistics.plotting.density.Kernel
import org.jetbrains.kotlinx.statistics.plotting.density.statDensity

fun densityPlot(
    sample: Iterable<Number>,
    weights: Iterable<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
): DensityPlot {
    return DensityPlot(statDensity(sample, weights, n, trim, adjust, kernel, fullScanMax, bandWidth),"x", weights != null)
}

fun DataFrame<*>.densityPlot(
    sample: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
): DensityPlot {
    return DensityPlot(statDensity(sample, weights,n, trim, adjust, kernel, fullScanMax, bandWidth, null), sample.name(), weights != null)
}

fun DataFrame<*>.densityPlot(
    x: String,
    weights: String? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
): DensityPlot {
    return DensityPlot(statDensity(x, weights,n, trim, adjust, kernel, fullScanMax, bandWidth, null), x, weights != null)
}


fun<T> DataFrame<T>.densityPlot(
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    selector: StatXNumericSelector<T>.() -> Unit
): DensityPlot {
    val appliedSelector = StatXNumericSelector(this).apply(selector)
    return DensityPlot(statDensity(n, trim, adjust, kernel, fullScanMax, bandWidth, null, selector), appliedSelector.x.name(), appliedSelector.weights != null)
}


fun DensityPlot.configure(block: DensityPlotContext.() -> Unit): Plot {
    return DensityPlotContext(this).apply {
        x(Stat.x)
        if (weighted) {
            y(Stat.densityWeighted)
        } else {
            y(Stat.density)
        }
        x.axis.name = name
        position = Position.dodge()
        block()
    }.toPlot()
}
