package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.api.GroupBy
import org.jetbrains.kotlinx.dataframe.api.concat
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinsOption
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelector
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.DensityGroupedPlotContext
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.DensityGroupedPlot
import org.jetbrains.kotlinx.statistics.plotting.density.BandWidth
import org.jetbrains.kotlinx.statistics.plotting.density.Kernel
import org.jetbrains.kotlinx.statistics.plotting.density.statDensity


fun <T, G> GroupBy<T, G>.densityPlot(
    x: String,
    weights: String? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
): DensityGroupedPlot<T> {
    return DensityGroupedPlot(
        statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth, null),
        x,
        weights != null
    )
}

fun <T, G> GroupBy<T, G>.densityPlot(
    x: ColumnAccessor<Number>,
    weight: ColumnAccessor<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
): DensityGroupedPlot<T> {
    return DensityGroupedPlot(
        statDensity(x, weight, n, trim, adjust, kernel, fullScanMax, bandWidth, null),
        x.name(),
        weight != null
    )
}


fun <T, G> GroupBy<T, G>.densityPlot(
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    selector: StatXNumericSelector<G>.() -> Unit
): DensityGroupedPlot<T> {
    val appliedSelector = StatXNumericSelector(this.groups.concat()).apply(selector)
    return DensityGroupedPlot(
        statDensity(n, trim, adjust, kernel, fullScanMax, bandWidth, null, selector),
        appliedSelector.x.name(),
        appliedSelector.weights != null
    )
}

fun <T> DensityGroupedPlot<T>.configure(block: DensityGroupedPlotContext<T>.() -> Unit): Plot {
    return DensityGroupedPlotContext(this).apply {
        x(Stat.x)
        if (weighted) {
            y(Stat.densityWeighted)
        } else {
            y(Stat.density)
        }
        key.columnNames().singleOrNull()?.let {
            fillColor(key[it])
        }
        //position = Position.dodge()
        x.axis.name = name
        block()
    }.toPlot()
}
