package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.layers.tiles
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.kandy.letsplot.x
import org.jetbrains.kotlinx.kandy.letsplot.y
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.HeatmapPlotContext
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.HeatmapPlot
import org.jetbrains.kotlinx.statistics.plotting.count.statCount
import org.jetbrains.kotlinx.statistics.plotting.count2d.statCount2D
import org.jetbrains.kotlinx.statistics.selectors.StatXSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXSelectorDSL
import org.jetbrains.kotlinx.statistics.selectors.StatXYSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXYSelectorDSL


inline fun <reified X, reified Y> heatmap(
    x: Iterable<X>,
    y: Iterable<Y>,
    weights: Iterable<Number>? = null,
): HeatmapPlot<X, Y> {
    return HeatmapPlot(statCount2D(x, y, weights), "x", "y", weights != null)
}

fun <X, Y> DataFrame<*>.heatmap(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Y>,
    weights: ColumnAccessor<Number>? = null,
): HeatmapPlot<X, Y> {
    return HeatmapPlot(statCount2D(x, y, weights), x.name(), y.name(), weights != null)
}

fun DataFrame<*>.heatmap(
    x: String,
    y: String,
    weights: String? = null,
): HeatmapPlot<Any?, Any?> {
    return HeatmapPlot(statCount2D(x, y, weights), x, y, weights != null)
}


fun <T, X, Y> DataFrame<T>.heatmap(
    selector: StatXYSelectorDSL<T, X, Y>
): HeatmapPlot<X, Y> {
    val appliedSelector = StatXYSelector<T, X, Y>(this).apply(selector)
    return HeatmapPlot(
        statCount2D(selector),
        appliedSelector.x.name(),
        appliedSelector.y.name(),
        appliedSelector.weights != null
    )
}

fun <X, Y> HeatmapPlot<X, Y>.configure(block: HeatmapPlotContext<X, Y>.() -> Unit): Plot {
    return HeatmapPlotContext(this).apply {
        x(Stat.x)
        y(Stat.y)
        if (weighted) {
            fillColor(Stat.countWeighted)
        } else {
            fillColor(Stat.count)
        }

        x.axis.name = xName
        y.axis.name = yName
        block()
    }.toPlot()
}
