package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinsOption
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelector
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.HistogramPlotContext
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.HistogramPlot
import org.jetbrains.kotlinx.statistics.plotting.bin.statBin


fun histogram(
    x: Iterable<Number>,
    weights: Iterable<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0)
): HistogramPlot {
    return HistogramPlot(statBin(x, weights, binsOption, binsAlign, null),"x", weights != null)
}

fun DataFrame<*>.histogram(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0)
): HistogramPlot {
    return HistogramPlot(statBin(x, weights, binsOption, binsAlign, null), x.name(), weights != null)
}

fun DataFrame<*>.histogram(
    xColumnName: String,
    weightsColumnName: String? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0)
): HistogramPlot {
    return HistogramPlot(statBin(xColumnName, weightsColumnName, binsOption, binsAlign, null), xColumnName, weightsColumnName != null)
}


fun<T> DataFrame<T>.histogram(
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    selector: StatXNumericSelector<T>.() -> Unit
): HistogramPlot {
    val appliedSelector = StatXNumericSelector(this).apply(selector)
    return HistogramPlot(statBin(binsOption, binsAlign, null, selector), appliedSelector.x.name(), appliedSelector.weights != null)
}


fun HistogramPlot.configure(block: HistogramPlotContext.() -> Unit): Plot {
    return HistogramPlotContext(this).apply {
        x(Stat.x)
        if (weighted) {
            y(Stat.countWeighted)
        } else {
            y(Stat.count)
        }
        x.axis.name = name
        position = Position.dodge()
        block()
    }.toPlot()
}
