package org.jetbrains.kotlinx.statistics.kandy.statplots

import org.jetbrains.kotlinx.dataframe.api.GroupBy
import org.jetbrains.kotlinx.dataframe.api.concat
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.ir.Plot
import org.jetbrains.kotlinx.kandy.letsplot.feature.Position
import org.jetbrains.kotlinx.kandy.letsplot.feature.position
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinsOption
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelector
import org.jetbrains.kotlinx.statistics.kandy.statplots.context.HistogramGroupedPlotContext
import org.jetbrains.kotlinx.statistics.kandy.statplots.models.HistogramGroupedPlot
import org.jetbrains.kotlinx.statistics.plotting.bin.statBin


fun<T, G> GroupBy<T, G>.histogram(
    sampleColumnName: String,
    weightsColumnName: String? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0)
): HistogramGroupedPlot<T> {
    return HistogramGroupedPlot(statBin(sampleColumnName, weightsColumnName, binsOption, binsAlign, null), sampleColumnName, weightsColumnName!= null)
}

fun<T, G> GroupBy<T, G>.histogram(
    sample: ColumnAccessor<Number>,
    weight: ColumnAccessor<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0)
): HistogramGroupedPlot<T> {
    return HistogramGroupedPlot(statBin(sample, weight, binsOption, binsAlign, null), sample.name(), weight != null)
}


fun<T, G> GroupBy<T, G>.histogram(
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    selector: StatXNumericSelector<G>.() -> Unit
): HistogramGroupedPlot<T> {
    val appliedSelector = StatXNumericSelector(this.groups.concat()).apply(selector)
    return HistogramGroupedPlot(statBin(binsOption, binsAlign, null, selector), appliedSelector.x.name(), appliedSelector.weights != null)
}

fun<T> HistogramGroupedPlot<T>.configure(block: HistogramGroupedPlotContext<T>.() -> Unit): Plot {
    return HistogramGroupedPlotContext(this).apply {
        x(Stat.x)
        if (weighted) {
            y(Stat.countWeighted)
        } else {
            y(Stat.count)
        }
        key.columnNames().singleOrNull()?.let {
            fillColor(key[it])
        }
        position = Position.dodge()
        x.axis.name = name
        block()
    }.toPlot()
}