package org.jetbrains.kotlinx.statistics.kandy.stattransform

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.kandy.letsplot.internal.*
import org.jetbrains.kotlinx.statistics.kandy.get
import org.jetbrains.kotlinx.statistics.kandy.stattransform.context.StatBoxplotLayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.boxplot.statBoxplot

public inline fun DataFramePlotBuilder<*>.statBoxplot(
    y: Iterable<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotLayerCreatorScope<String>.() -> Unit
) {
    val statBoxplotFrame = statBoxplot(y, whiskerIQRRatio)
    StatBoxplotLayerCreatorScope(statBoxplotFrame, this).apply(block)
}

public inline fun DataFramePlotBuilder<*>.statBoxplot(
    y: DataColumn<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotLayerCreatorScope<String>.() -> Unit
) {
    val statBoxplotFrame = statBoxplot(y, whiskerIQRRatio)
    StatBoxplotLayerCreatorScope(statBoxplotFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public inline fun DataFramePlotBuilder<*>.statBoxplot(
    y: ColumnAccessor<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotLayerCreatorScope<String>.() -> Unit
) {
    val statBoxplotFrame = statBoxplot(datasetBuilderImpl.get(y), whiskerIQRRatio)
    StatBoxplotLayerCreatorScope(statBoxplotFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public inline fun<T> DataFramePlotBuilder<*>.statBoxplot(
    y: String,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotLayerCreatorScope<String>.() -> Unit
) {
    val statBoxplotFrame = internalDataFrame.statBoxplot(y, whiskerIQRRatio)
    StatBoxplotLayerCreatorScope(statBoxplotFrame, this).apply(block)
}

public inline fun<T> DataFramePlotBuilder<*>.statBoxplot(
    x: DataColumn<T>,
    y: DataColumn<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotLayerCreatorScope<T>.() -> Unit
) {
    val statBoxplotFrame = statBoxplot(x, y, whiskerIQRRatio)
    StatBoxplotLayerCreatorScope(statBoxplotFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public inline fun<T> DataFramePlotBuilder<*>.statBoxplot(
    x: ColumnAccessor<T>,
    y: ColumnAccessor<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotLayerCreatorScope<T>.() -> Unit
) {
    val statBoxplotFrame = datasetBuilderImpl.baseDataFrame.statBoxplot(x, y, whiskerIQRRatio)
    StatBoxplotLayerCreatorScope(statBoxplotFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public inline fun DataFramePlotBuilder<*>.statBoxplot(
    x: String,
    y: String,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotLayerCreatorScope<Any?>.() -> Unit
) {
    val statBoxplotFrame = datasetBuilderImpl.baseDataFrame.statBoxplot(column<Any?>(x), column(y), whiskerIQRRatio)
    StatBoxplotLayerCreatorScope(statBoxplotFrame, this).apply(block)
}
