package org.jetbrains.kotlinx.statistics.kandy.stattransform

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.kandy.letsplot.internal.*
import org.jetbrains.kotlinx.statistics.kandy.get
import org.jetbrains.kotlinx.statistics.kandy.stattransform.context.StatBoxplotOutliersLayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.statBoxplotOutliers

public inline fun DataFramePlotBuilder<*>.statBoxplotOutliers(
    y: Iterable<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotOutliersLayerCreatorScope<String>.() -> Unit
) {
    val statBoxplotOutliersFrame = statBoxplotOutliers(y, whiskerIQRRatio)
    StatBoxplotOutliersLayerCreatorScope(statBoxplotOutliersFrame, this).apply(block)
}

public inline fun DataFramePlotBuilder<*>.statBoxplotOutliers(
    y: DataColumn<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotOutliersLayerCreatorScope<String>.() -> Unit
) {
    val statBoxplotOutliersFrame = statBoxplotOutliers(y, whiskerIQRRatio)
    StatBoxplotOutliersLayerCreatorScope(statBoxplotOutliersFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public inline fun DataFramePlotBuilder<*>.statBoxplotOutliers(
    y: ColumnAccessor<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotOutliersLayerCreatorScope<String>.() -> Unit
) {
    val statBoxplotOutliersFrame = statBoxplotOutliers(datasetBuilderImpl.get(y), whiskerIQRRatio)
    StatBoxplotOutliersLayerCreatorScope(statBoxplotOutliersFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public inline fun<T> DataFramePlotBuilder<*>.statBoxplotOutliers(
    y: String,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotOutliersLayerCreatorScope<String>.() -> Unit
) {
    val statBoxplotOutliersFrame = internalDataFrame.statBoxplotOutliers(y, whiskerIQRRatio)
    StatBoxplotOutliersLayerCreatorScope(statBoxplotOutliersFrame, this).apply(block)
}

public inline fun<T> DataFramePlotBuilder<*>.statBoxplotOutliers(
    x: DataColumn<T>,
    y: DataColumn<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotOutliersLayerCreatorScope<T>.() -> Unit
) {
    val statBoxplotOutliersFrame = statBoxplotOutliers(x, y, whiskerIQRRatio)
    StatBoxplotOutliersLayerCreatorScope(statBoxplotOutliersFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public inline fun<T> DataFramePlotBuilder<*>.statBoxplotOutliers(
    x: ColumnAccessor<T>,
    y: ColumnAccessor<Number>,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotOutliersLayerCreatorScope<T>.() -> Unit
) {
    val statBoxplotOutliersFrame = datasetBuilderImpl.baseDataFrame.statBoxplotOutliers(x, y, whiskerIQRRatio)
    StatBoxplotOutliersLayerCreatorScope(statBoxplotOutliersFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public inline fun DataFramePlotBuilder<*>.statBoxplotOutliers(
    x: String,
    y: String,
    whiskerIQRRatio: Double = 1.5,
    block: StatBoxplotOutliersLayerCreatorScope<Any?>.() -> Unit
) {
    val statBoxplotOutliersFrame = datasetBuilderImpl.baseDataFrame.statBoxplotOutliers(column<Any?>(x), column(y), whiskerIQRRatio)
    StatBoxplotOutliersLayerCreatorScope(statBoxplotOutliersFrame, this).apply(block)
}
