package org.jetbrains.kotlinx.statistics.kandy.stattransform

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.statistics.kandy.stattransform.context.StatCountLayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.count.statCount

@Suppress("INVISIBLE_MEMBER")
public fun<X> DataFramePlotBuilder<*>.statCount(
    x: Iterable<X>,
    weights: Iterable<Number>? = null,
    block: StatCountLayerCreatorScope<X>.() -> Unit
) {
    val statCountFrame = statCount(x, weights)
    StatCountLayerCreatorScope(statCountFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun<X> DataFramePlotBuilder<*>.statCount(
    x: DataColumn<X>,
    weights: DataColumn<Number>? = null,
    block: StatCountLayerCreatorScope<X>.() -> Unit
) {
    val statCountFrame = statCount(x, weights)
    StatCountLayerCreatorScope(statCountFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public  fun<X> DataFramePlotBuilder<*>.statCount(
    x: ColumnAccessor<X>,
    weights: ColumnAccessor<Number>? = null,
    block: StatCountLayerCreatorScope<X>.() -> Unit
) {
    val statCountFrame = internalDataFrame.statCount(x, weights)
    StatCountLayerCreatorScope(statCountFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statCount(
    x: String,
    weights: String? = null,
    block: StatCountLayerCreatorScope<Any?>.() -> Unit
) {
    val statCountFrame = internalDataFrame.statCount(x, weights)
    StatCountLayerCreatorScope(statCountFrame, this).apply(block)
}
