package org.jetbrains.kotlinx.statistics.kandy.stattransform

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.statistics.kandy.stattransform.context.StatCount2DLayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.count2d.statCount2D

@Suppress("INVISIBLE_MEMBER")
public inline fun<reified X, reified Y> DataFramePlotBuilder<*>.statCount2D(
    x: Iterable<X>,
    y: Iterable<Y>,
    weights: Iterable<Number>? = null,
    block: StatCount2DLayerCreatorScope<X, Y>.() -> Unit
) {
    val statCount2DFrame = statCount2D(x, y, weights)
    StatCount2DLayerCreatorScope(statCount2DFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun<X, Y> DataFramePlotBuilder<*>.statCount2D(
    x: DataColumn<X>,
    y: DataColumn<Y>,
    weights: DataColumn<Number>? = null,
    block: StatCount2DLayerCreatorScope<X, Y>.() -> Unit
) {
    val statCount2DFrame = statCount2D(x, y, weights)
    StatCount2DLayerCreatorScope(statCount2DFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public  fun<X, Y> DataFramePlotBuilder<*>.statCount2D(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Y>,
    weights: ColumnAccessor<Number>? = null,
    block: StatCount2DLayerCreatorScope<X, Y>.() -> Unit
) {
    val statCount2DFrame = internalDataFrame.statCount2D(x, y, weights)
    StatCount2DLayerCreatorScope(statCount2DFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statCount2D(
    x: String,
    y: String,
    weights: String? = null,
    block: StatCount2DLayerCreatorScope<Any?, Any?>.() -> Unit
) {
    val statCount2DFrame = internalDataFrame.statCount2D(x, y, weights)
    StatCount2DLayerCreatorScope(statCount2DFrame, this).apply(block)
}
