package org.jetbrains.kotlinx.statistics.kandy.stattransform

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.statistics.kandy.stattransform.context.StatDensityLayerCreatorScope
import org.jetbrains.kotlinx.statistics.plotting.density.statDensity
import org.jetbrains.kotlinx.statistics.kandy.get
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.density.BandWidth
import org.jetbrains.kotlinx.statistics.plotting.density.Kernel

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statDensity(
    x: Iterable<Number>,
    weights: Iterable<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: StatDensityLayerCreatorScope.() -> Unit
) {
    val statDensityFrame = statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth)
    StatDensityLayerCreatorScope(statDensityFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statDensity(
    x: DataColumn<Number>,
    weights: DataColumn<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: StatDensityLayerCreatorScope.() -> Unit
) {
    val statDensityFrame = statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth)
    StatDensityLayerCreatorScope(statDensityFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statDensity(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: StatDensityLayerCreatorScope.() -> Unit
) {
    val statDensityFrame = internalDataFrame.statDensity(
        x,
        weights,
        n,
        trim,
        adjust,
        kernel,
        fullScanMax,
        bandWidth
    )
    StatDensityLayerCreatorScope(statDensityFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statDensity(
    x: String,
    weights: String? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: StatDensityLayerCreatorScope.() -> Unit
) {
    val statDensityFrame = internalDataFrame.statDensity(
        x,
        weights,
        n,
        trim,
        adjust,
        kernel,
        fullScanMax,
        bandWidth
    )
    StatDensityLayerCreatorScope(statDensityFrame, this).apply(block)
}
