package org.jetbrains.kotlinx.statistics.kandy.stattransform

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.GroupBy
import org.jetbrains.kotlinx.dataframe.api.column
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.GroupedDataScope
import org.jetbrains.kotlinx.kandy.dsl.internal.LayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.stattransform.context.StatDensityGroupedLayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.util.internalGroupBy
import org.jetbrains.kotlinx.statistics.plotting.density.BandWidth
import org.jetbrains.kotlinx.statistics.plotting.density.Kernel
import org.jetbrains.kotlinx.statistics.plotting.density.statDensity

@Suppress("INVISIBLE_MEMBER")
public fun <T> GroupedDataScope<T, *>.statDensity(
    x: DataColumn<Number>,
    weights: DataColumn<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: StatDensityGroupedLayerCreatorScope<T>.() -> Unit
) {
    val groupBy = internalGroupBy as GroupBy<T, *>
    val statDensityGrouped = groupBy.statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth)
    StatDensityGroupedLayerCreatorScope(statDensityGrouped, this as LayerCreatorScope).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public  fun <T> GroupedDataScope<T, *>.statDensity(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: StatDensityGroupedLayerCreatorScope<T>.() -> Unit
) {
    val groupBy =  internalGroupBy as GroupBy<T, *>
    val statDensityGrouped = groupBy.statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth)
    StatDensityGroupedLayerCreatorScope(statDensityGrouped, this as LayerCreatorScope).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun <T> GroupedDataScope<T, *>.statDensity(
    x: String,
    weights: String? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    block: StatDensityGroupedLayerCreatorScope<T>.() -> Unit
) {
   return this.statDensity(column(x),weights?.let { column(it) }, n, trim, adjust, kernel, fullScanMax, bandWidth, block)
}
