package org.jetbrains.kotlinx.statistics.kandy.stattransform

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.DataFramePlotBuilder
import org.jetbrains.kotlinx.statistics.kandy.stattransform.context.StatSmoothLayerCreatorScope
import org.jetbrains.kotlinx.statistics.kandy.util.internalDataFrame
import org.jetbrains.kotlinx.statistics.plotting.smooth.SmoothMethod
import org.jetbrains.kotlinx.statistics.plotting.smooth.statSmooth

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statSmooth(
    x: Iterable<Number>,
    y: Iterable<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: StatSmoothLayerCreatorScope.() -> Unit
) {
    val statSmoothFrame = statSmooth(x, y, method, smootherPointCount)
    StatSmoothLayerCreatorScope(statSmoothFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statSmooth(
    x: DataColumn<Number>,
    y: DataColumn<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: StatSmoothLayerCreatorScope.() -> Unit
) {
    val statSmoothFrame = statSmooth(x, y, method, smootherPointCount)
    StatSmoothLayerCreatorScope(statSmoothFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statSmooth(
    x: ColumnAccessor<Number>,
    y: ColumnAccessor<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: StatSmoothLayerCreatorScope.() -> Unit
) {
    val statSmoothFrame = internalDataFrame.statSmooth(
        x, y, method, smootherPointCount
    )
    StatSmoothLayerCreatorScope(statSmoothFrame, this).apply(block)
}

@Suppress("INVISIBLE_MEMBER")
public fun DataFramePlotBuilder<*>.statSmooth(
    x: String,
    y: String,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    block: StatSmoothLayerCreatorScope.() -> Unit
) {
    val statSmoothFrame= internalDataFrame.statSmooth(
        x, y, method, smootherPointCount
    )
    StatSmoothLayerCreatorScope(statSmoothFrame, this).apply(block)
}
