package org.jetbrains.kotlinx.statistics

import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.DataFrameBuilder
import org.jetbrains.kotlinx.dataframe.api.dataFrameOf
import org.jetbrains.kotlinx.dataframe.api.median
import org.jetbrains.kotlinx.dataframe.io.readCSV
import org.jetbrains.kotlinx.kandy.dsl.plot
import org.jetbrains.kotlinx.kandy.letsplot.layers.hLine
import org.jetbrains.kotlinx.kandy.letsplot.layers.points
import org.jetbrains.kotlinx.kandy.letsplot.layers.vLine
import org.jetbrains.kotlinx.kandy.letsplot.settings.LineType
import org.jetbrains.kotlinx.kandy.util.color.Color
import org.jetbrains.kotlinx.statistics.kandy.layers.histogram
import org.jetbrains.kotlinx.statistics.kandy.layers.smoothLine
import org.jetbrains.kotlinx.statistics.plotting.bin.statBin
import org.jetbrains.kotlinx.statistics.plotting.smooth.SmoothMethod


fun main() {

    dataFrameOf("d" to listOf(1)).statBin("").Stat.x

     val mpgDf =
        DataFrame.readCSV("https://raw.githubusercontent.com/JetBrains/lets-plot-kotlin/master/docs/examples/data/mpg.csv")

    val medianColor = Color.hex("#756bb1")
     val ctyMedian = mpgDf.median("cty")
     val hwyMedian = mpgDf.median("hwy")
    mpgDf.plot {
        points {
            x("cty")
            y("hwy")
        }
        vLine {
            xIntercept.constant(ctyMedian)
            color = medianColor
            type = LineType.DASHED
        }
        hLine {
            yIntercept.constant(hwyMedian)
            color = medianColor
            type = LineType.DASHED
        }
        smoothLine("cty", "hwy", method = SmoothMethod.Linear()) {
            color = Color.hex("#de2d26")
        }
    }
}
