package org.jetbrains.kotlinx.statistics.plotting.bin

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.*
import org.jetbrains.kotlinx.statistics.binning.BinsAlign
import org.jetbrains.kotlinx.statistics.binning.BinsOption
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelector
import org.jetbrains.kotlinx.statistics.plotting.bin.models.StatBinGroupBy
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*


@Suppress("INVISIBLE_MEMBER")
fun <T, G> GroupBy<T, G>.statBin(
    x: DataColumn<Number>,
    weights: DataColumn<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    binsRange: Pair<Number, Number>? = null,
): StatBinGroupBy<T> {
    val keyNames = keys.columnNames()
    val newGroupBy = (concatFixed().getColumns(*keyNames.toTypedArray()) + x).toDataFrame().groupBy(*keyNames.toTypedArray())
    return newGroupBy.statBin(x.toColumnAccessor(), weights?.toColumnAccessor(), binsOption, binsAlign, binsRange) as
            StatBinGroupBy<T>
}

// todo other selecting options
fun<T, G> GroupBy<T, G>.statBin(
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    binsRange: Pair<Number, Number>? = null,
    selector: StatXNumericSelector<G>.() -> Unit,
): StatBinGroupBy<T> {
    val appliedSelector = StatXNumericSelector(groups.concat()).apply(selector)
    return statBin(appliedSelector.x, appliedSelector.weights, binsOption, binsAlign, binsRange)
}



fun <T, G> GroupBy<T, G>.statBin(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    binsRange: Pair<Number, Number>? = null,
): StatBinGroupBy<T> {
    val range = binsRange ?: groups.concat()[x].values().map {
        it.toDouble()
    }.let { it.min() to it.max() }
    return StatBinGroupBy(updateGroups {
        statBin(x, weights, binsOption, binsAlign, range)
    })
}

fun <T, G> GroupBy<T, G>.statBin(
    x: String,
    weights: String? = null,
    binsOption: BinsOption = BinsOption.byNumber(20),
    binsAlign: BinsAlign = BinsAlign.center(0.0),
    binsRange: Pair<Number, Number>? = null,
): StatBinGroupBy<T> {
    return statBin(
        column(x),
        weights?.let { column(it) },
        binsOption,
        binsAlign,
        binsRange
    )
}
