package org.jetbrains.kotlinx.statistics.plotting.boxplot

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.*
import org.jetbrains.kotlinx.statistics.plotting.boxplot.models.StatBoxplotGroupBy
import org.jetbrains.kotlinx.statistics.selectors.StatXYNumSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXYNumSelectorDSL
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*

@Suppress("INVISIBLE_MEMBER")
fun <T, X> GroupBy<T, *>.statBoxplot(
    x: DataColumn<X>,
    y: DataColumn<Number>,
    whiskerIQRRatio: Double = 1.5
): StatBoxplotGroupBy<T, X> {
    val keyNames = keys.columnNames()
    val newGroupBy = (concatFixed().getColumns(*keyNames.toTypedArray()) + x + y).toDataFrame().groupBy(*keyNames.toTypedArray())
    return newGroupBy.statBoxplot(x.toColumnAccessor(), y.toColumnAccessor(), whiskerIQRRatio) as
            StatBoxplotGroupBy<T, X>
}

// todo other selecting options
fun<T, G, X> GroupBy<T, G>.statBoxplot(
    whiskerIQRRatio: Double = 1.5,
    selector: StatXYNumSelectorDSL<G, X>,
): StatBoxplotGroupBy<T, X> {
    val appliedSelector = StatXYNumSelector<G, X>(groups.concat()).apply(selector)
    return statBoxplot(appliedSelector.x, appliedSelector.y, whiskerIQRRatio)
}



fun <T, X> GroupBy<T, *>.statBoxplot(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Number>,
    whiskerIQRRatio: Double = 1.5,
): StatBoxplotGroupBy<T, X> {
    return StatBoxplotGroupBy(updateGroups {
        statBoxplot(x, y, whiskerIQRRatio)
    })
}

fun <T> GroupBy<T, *>.statBoxplot(
    x: String,
    y: String,
    whiskerIQRRatio: Double = 1.5,
): StatBoxplotGroupBy<T, Any?> {
    return statBoxplot(
        column(x),
        column(y),
        whiskerIQRRatio
    )
}
