package org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.statistics.math3.stat.descriptive.DescriptiveStatistics
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.BoxplotOutliersStatistics
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.StatBoxplotOutliersFrame

private fun<T> outliers(x: T, sample: Iterable<Number>, whiskerIQRRatio: Double): List<BoxplotOutliersStatistics<T>> {
    val sampleDouble = sample.map {it.toDouble()}
    val stats = DescriptiveStatistics(sample.map {it.toDouble()}.toDoubleArray())

    val lowerHinge = stats.getPercentile(25.0)
    val upperHinge = stats.getPercentile(75.0)
    val IQR = upperHinge - lowerHinge
    val lowerFence = lowerHinge - IQR * whiskerIQRRatio
    val upperFence = upperHinge + IQR * whiskerIQRRatio

    val outliers = sampleDouble.filter { y -> y < lowerFence || y > upperFence }.map {
        BoxplotOutliersStatistics(x, it)
    }

    return outliers
}

fun statBoxplotOutliers(y: Iterable<Number>, whiskerIQRRatio: Double = 1.5, name: String = ""): StatBoxplotOutliersFrame<String> {
    val outliers = outliers(name, y, whiskerIQRRatio)
    return StatBoxplotOutliersFrame.fromList(listOf( outliers))
}

fun statBoxplotOutliers(y: DataColumn<Number>, whiskerIQRRatio: Double = 1.5): StatBoxplotOutliersFrame<String> {
    return statBoxplotOutliers(y.asIterable(), whiskerIQRRatio, y.name())
}

fun <T> DataFrame<T>.statBoxplotOutliers(y: ColumnAccessor<Number>, whiskerIQRRatio: Double = 1.5): StatBoxplotOutliersFrame<String> {
    return statBoxplotOutliers(get(y), whiskerIQRRatio)
}

fun <T> DataFrame<T>.statBoxplotOutliers(y:String, whiskerIQRRatio: Double = 1.5): StatBoxplotOutliersFrame<String> {
    return statBoxplotOutliers(column(y), whiskerIQRRatio)
}

fun <T, X> DataFrame<T>.statBoxplotOutliers(x: ColumnAccessor<X>, y: ColumnAccessor<Number>, whiskerIQRRatio: Double = 1.5): StatBoxplotOutliersFrame<X> {
    val outliers = groupBy(x).map {
        val xs = key[x]
        val ys = it.group[y].toList()
        outliers(xs, ys, whiskerIQRRatio)
    }
    return StatBoxplotOutliersFrame.fromList(outliers)
}

fun <T> DataFrame<T>.statBoxplotOutliers(x: String, y: String, whiskerIQRRatio: Double = 1.5): StatBoxplotOutliersFrame<Any?> {
    return statBoxplotOutliers(column(x), column(y), whiskerIQRRatio)
}

fun<T> statBoxplotOutliers(x: DataColumn<T>, y: DataColumn<Number>, whiskerIQRRatio: Double = 1.5): StatBoxplotOutliersFrame<T> {
    return dataFrameOf(x, y).statBoxplotOutliers(x.toColumnAccessor(), y.toColumnAccessor(), whiskerIQRRatio)
}
