package org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.*
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.StatBoxplotOutliers
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.StatBoxplotOutliersFrame
import org.jetbrains.kotlinx.statistics.plotting.boxplotOutliers.models.StatBoxplotOutliersGroupBy
import org.jetbrains.kotlinx.statistics.selectors.StatXYNumSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXYNumSelectorDSL
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*

@Suppress("INVISIBLE_MEMBER")
fun <T, X> GroupBy<T, *>.statBoxplotOutliers(
    x: DataColumn<X>,
    y: DataColumn<Number>,
    whiskerIQRRatio: Double = 1.5
): StatBoxplotOutliersGroupBy<T, X> {
    val keyNames = keys.columnNames()
    val newGroupBy =
        (concatFixed().getColumns(*keyNames.toTypedArray()) + x + y).toDataFrame().groupBy(*keyNames.toTypedArray())
    return newGroupBy.statBoxplotOutliers(x.toColumnAccessor(), y.toColumnAccessor(), whiskerIQRRatio) as
            StatBoxplotOutliersGroupBy<T, X>
}

// todo other selecting options
fun <T, G, X> GroupBy<T, G>.statBoxplotOutliers(
    whiskerIQRRatio: Double = 1.5,
    selector: StatXYNumSelectorDSL<G, X>,
): StatBoxplotOutliersGroupBy<T, X> {
    val appliedSelector = StatXYNumSelector<G, X>(groups.concat()).apply(selector)
    return statBoxplotOutliers(appliedSelector.x, appliedSelector.y, whiskerIQRRatio)
}


fun <T, X> GroupBy<T, *>.statBoxplotOutliers(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Number>,
    whiskerIQRRatio: Double = 1.5,
): StatBoxplotOutliersGroupBy<T, X> {
    return StatBoxplotOutliersGroupBy(updateGroups {
        val frame = statBoxplotOutliers(x, y, whiskerIQRRatio)
        frame
    })
}

fun <T> GroupBy<T, *>.statBoxplotOutliers(
    x: String,
    y: String,
    whiskerIQRRatio: Double = 1.5,
): StatBoxplotOutliersGroupBy<T, Any?> {
    return statBoxplotOutliers(
        column(x),
        column(y),
        whiskerIQRRatio
    )
}
