package org.jetbrains.kotlinx.statistics.plotting.candlestick

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.dataframe.columns.ColumnGroup
import org.jetbrains.kotlinx.dataframe.impl.asList
import org.jetbrains.kotlinx.statistics.plotting.candlestick.models.CandlestickStatistics
import org.jetbrains.kotlinx.statistics.plotting.candlestick.models.StatCandlestick
import org.jetbrains.kotlinx.statistics.plotting.candlestick.models.StatCandlestickFrame
import org.jetbrains.kotlinx.statistics.plotting.count2d.models.StatCount2D
import org.jetbrains.kotlinx.statistics.plotting.count2d.models.StatCount2DFrame
import org.jetbrains.kotlinx.statistics.plotting.density.models.DensityStatistics
import org.jetbrains.kotlinx.statistics.plotting.density.models.StatDensity
import org.jetbrains.kotlinx.statistics.selectors.StatCandlestickSelector
import org.jetbrains.kotlinx.statistics.selectors.StatCandlestickSelectorDSL
import org.jetbrains.kotlinx.statistics.selectors.StatXYSelector
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*

fun<X> statCandlestick(
    x: Iterable<X>,
    open: Iterable<Number>,
    high: Iterable<Number>,
    low: Iterable<Number>,
    close: Iterable<Number>
): StatCandlestickFrame<X> {
    val lower = mutableListOf<Number>()
    val upper = mutableListOf<Number>()
    val isIncreased = mutableListOf<Boolean>()

    for ((openVal, closeVal) in open.zip(close)) {
        if (openVal.toDouble() >= closeVal.toDouble()) {
            lower.add(closeVal)
            upper.add(openVal)
            isIncreased.add(false)
        } else {
            lower.add(openVal)
            upper.add(closeVal)
            isIncreased.add(true)
        }
    }

    return StatCandlestickFrame(
        dataFrameOf(
            dataFrameOf(
                "x" to x.asList(),
                "open" to open.asList(),
                "close" to close.asList(),
                "min" to low.asList(),
                "lower" to lower,
                "upper" to upper,
                "max" to high.asList(),
                "isIncreased" to isIncreased,
            ).asColumnGroup("Stat") as ColumnGroup<CandlestickStatistics<X>>
        ) as DataFrame<StatCandlestick<X>>
    )
}

fun <T> DataFrame<T>.statCandlestick(
    x: String,
    open: String,
    high: String,
    low: String,
    close: String,
): StatCandlestickFrame<Any?> {
    return statCandlestick(column(x), column(open), column(high), column(low), column(close))
}

fun<T, X> DataFrame<T>.statCandlestick(
    x: ColumnAccessor<X>,
    open: ColumnAccessor<Number>,
    high: ColumnAccessor<Number>,
    low: ColumnAccessor<Number>,
    close: ColumnAccessor<Number>,
): StatCandlestickFrame<X>  {
    return statCandlestick(get(x), get(open), get(high), get(low), get(close))
}

fun<X> statCandlestick(
    x: DataColumn<X>,
    open: DataColumn<Number>,
    high: DataColumn<Number>,
    low: DataColumn<Number>,
    close: DataColumn<Number>,
): StatCandlestickFrame<X>  {
    return statCandlestick(x.asIterable(), open.asIterable(), high.asIterable(), low.asIterable(), close.asIterable())
}

fun<T, X> DataFrame<T>.statCandlestick(
    selector: StatCandlestickSelectorDSL<T, X>
): StatCandlestickFrame<X>  {
    val appliedSelector = StatCandlestickSelector<T,X>(this).apply(selector)
    return statCandlestick(appliedSelector.x, appliedSelector.open, appliedSelector.high, appliedSelector.low,
        appliedSelector.close)
}
