package org.jetbrains.kotlinx.statistics.plotting.count

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.dataframe.impl.asList
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelectorDSL
import org.jetbrains.kotlinx.statistics.plotting.count.models.StatCount
import org.jetbrains.kotlinx.statistics.plotting.count.models.StatCountFrame
import org.jetbrains.kotlinx.statistics.selectors.StatXSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXSelectorDSL

fun<T> statCount(
    x: Iterable<T>,
    weights: Iterable<Number>? = null
): StatCountFrame<T> {
    val valuesList = x.asList()
    val weightsDouble = weights?.map { it.toDouble() } ?: List(valuesList.size) { 1.0 }
    val newDF = dataFrameOf("x" to valuesList,"weights" to weightsDouble)
    return StatCountFrame(dataFrameOf(
        newDF.groupBy("x").aggregate {
            val count = count()
            count into "count"
            // todo fix in df: empty df sum
            val countWeighted = if ("weights" in columnNames()) {
                sum("weights").toDouble()
            } else {
                count.toDouble()
            }
            countWeighted into "countWeighted"
        }.asColumnGroup("Stat")
    ) as DataFrame<StatCount<T>>)
}

fun<T, R> DataFrame<T>.statCount(
    selector: StatXSelectorDSL<T, R>
): StatCountFrame<R> {
    val appliedSelector = StatXSelector<T, R>(this).apply(selector)
    val x = appliedSelector.x
    val weights = appliedSelector.weights
    return statCount(x, weights)
}

fun<T> statCount(
    x: DataColumn<T>,
    weights: DataColumn<Number>? = null
): StatCountFrame<T> {
    return statCount(x.asIterable(), weights?.asIterable())
}

fun<T, R> DataFrame<T>.statCount(
    sample: ColumnAccessor<R>,
    weights: ColumnAccessor<Number>? = null,
): StatCountFrame<R> {
    return statCount(get(sample), weights?.let { get(it) })
}

fun<T> DataFrame<T>.statCount(
    sample: String,
    weights: String? = null,
): StatCountFrame<Any?>  {
    return statCount(column(sample), weights?.let { column(it) })
}
