package org.jetbrains.kotlinx.statistics.plotting.count

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*
import org.jetbrains.kotlinx.statistics.plotting.count.models.StatCountGroupBy
import org.jetbrains.kotlinx.statistics.selectors.StatXSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXSelectorDSL

@Suppress("INVISIBLE_MEMBER")
fun <T, G, X> GroupBy<T, G>.statCount(
    sample: DataColumn<X>,
    weights: DataColumn<Number>? = null,
): StatCountGroupBy<T, X> {
    val keyNames = keys.columnNames()
    val newGroupBy = (concatFixed().getColumns(*keyNames.toTypedArray()) + sample).toDataFrame().groupBy(*keyNames.toTypedArray())
    return newGroupBy.statCount(sample.toColumnAccessor(), weights?.toColumnAccessor()) as
            StatCountGroupBy<T, X>
}

fun<T, G, X> GroupBy<T, G>.statCount(
    selector: StatXSelectorDSL<G, X>,
): StatCountGroupBy<T, X> {
    val appliedSelector = StatXSelector<G, X>(groups.concat()).apply(selector)
    return statCount(appliedSelector.x, appliedSelector.weights)
}

fun <T, G, X> GroupBy<T, G>.statCount(
    sample: ColumnAccessor<X>,
    weights: ColumnAccessor<Number>? = null,
): StatCountGroupBy<T, X> {
    return StatCountGroupBy(updateGroups {
        statCount(sample, weights)
    })
}

fun <T, G> GroupBy<T, G>.statCount(
    sample: String,
    weights: String? = null,
): StatCountGroupBy<T, Any?> {
    return statCount(
        column(sample),
        weights?.let { column(it) },
    )
}
