package org.jetbrains.kotlinx.statistics.plotting.count2d

import org.jetbrains.kotlinx.dataframe.ColumnSelector
import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.dataframe.impl.asList
import org.jetbrains.kotlinx.statistics.plotting.count2d.models.StatCount2D
import org.jetbrains.kotlinx.statistics.plotting.count2d.models.StatCount2DFrame
import org.jetbrains.kotlinx.statistics.selectors.StatXYSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXYSelectorDSL

inline fun<reified X, reified Y> statCount2D(
    x: Iterable<X>,
    y: Iterable<Y>,
    weights: Iterable<Number>? = null
): StatCount2DFrame<X, Y> {
   return statCount2D(
       x.toColumn("x", infer = Infer.Type),
       y.toColumn("y", infer = Infer.Type),
       weights?.toColumn("weights", infer = Infer.Type)
   )
}

fun<T> DataFrame<T>.statCount2D(
    x: String,
    y: String,
    weights: String? = null
): StatCount2DFrame<Any?, Any?> {
    return statCount2D(column(x), column(y), weights?.let { column(it) })
}

fun<T, X, Y> DataFrame<T>.statCount2D(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Y>,
    weights: ColumnAccessor<Number>? = null
): StatCount2DFrame<X, Y> {
    return StatCount2DFrame(dataFrameOf(
        groupBy(get(x) named "x", get(y) named "y").aggregate {
            val count = count()
            count into "count"
            val countWeighted = if (weights != null) {
                sum(weights)
            } else {
                count
            }
            countWeighted into "countWeighted"
        }.asColumnGroup("Stat")
    ) as DataFrame<StatCount2D<X, Y>>)
}

fun<X, Y> statCount2D(
    x: DataColumn<X>,
    y: DataColumn<Y>,
    weights: DataColumn<Number>? = null
): StatCount2DFrame<X, Y> {
    return dataFrameOf(x, y, weights ?: List(x.size()) {1.0}.toColumn()).statCount2D(x.toColumnAccessor(), y.toColumnAccessor(), weights?.toColumnAccessor())
}

fun<T, X, Y> DataFrame<T>.statCount2D(
    selector: StatXYSelectorDSL<T, X, Y>
): StatCount2DFrame<X, Y> {
    val appliedSelector = StatXYSelector<T,X,Y>(this).apply(selector)
    return statCount2D(appliedSelector.x, appliedSelector.y, appliedSelector.weights)
}
