package org.jetbrains.kotlinx.statistics.plotting.count2d

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.*
import org.jetbrains.kotlinx.statistics.plotting.count2d.models.StatCount2DGroupBy
import org.jetbrains.kotlinx.statistics.selectors.StatXYSelector
import org.jetbrains.kotlinx.statistics.selectors.StatXYSelectorDSL
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*

@Suppress("INVISIBLE_MEMBER")
fun <T, X, Y> GroupBy<T, *>.statCount2D(
    x: DataColumn<X>,
    y: DataColumn<Y>,
    weights: DataColumn<Number>? = null,
): StatCount2DGroupBy<T, X, Y> {
    val keyNames = keys.columnNames()
    val newGroupBy = (concatFixed().getColumns(*keyNames.toTypedArray()) + x + y).toDataFrame().groupBy(*keyNames.toTypedArray())
    return newGroupBy.statCount2D(x.toColumnAccessor(), y.toColumnAccessor(), weights?.toColumnAccessor()) as
            StatCount2DGroupBy<T, X, Y>
}

fun<T, G, X, Y> GroupBy<T, G>.statCount2D(
    selector: StatXYSelectorDSL<G, X, Y>,
): StatCount2DGroupBy<T, X, Y> {
    val appliedSelector = StatXYSelector<G, X, Y>(groups.concat()).apply(selector)
    return statCount2D(appliedSelector.x, appliedSelector.y, appliedSelector.weights)
}

fun <T, G, X, Y> GroupBy<T, G>.statCount2D(
    x: ColumnAccessor<X>,
    y: ColumnAccessor<Y>,
    weights: ColumnAccessor<Number>? = null,
): StatCount2DGroupBy<T, X, Y> {
    return StatCount2DGroupBy(updateGroups {
        statCount2D(x,y, weights)
    })
}

fun <T, G> GroupBy<T, G>.statCount2D(
    x: String,
    y: String,
    weights: String? = null,
): StatCount2DGroupBy<T, Any?, Any?> {
    return statCount2D(
        column(x),
        column(y),
        weights?.let { column(it) },
    )
}
