package org.jetbrains.kotlinx.statistics.plotting.density

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.*
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*
import org.jetbrains.kotlinx.statistics.selectors.StatXNumericSelector
import org.jetbrains.kotlinx.statistics.plotting.density.models.StatDensityGroupBy

@Suppress("INVISIBLE_MEMBER")
fun <T, G> GroupBy<T, G>.statDensity(
    x: DataColumn<Number>,
    weights: DataColumn<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    range: Pair<Number, Number>? = null,
): StatDensityGroupBy<T> {
    val keyNames = keys.columnNames()
    val newGroupBy =
        (concatFixed().getColumns(*keyNames.toTypedArray()) + x).toDataFrame().groupBy(*keyNames.toTypedArray())
    return newGroupBy.statDensity(
        x.toColumnAccessor(),
        weights?.toColumnAccessor(),
        n,
        trim,
        adjust,
        kernel,
        fullScanMax,
        bandWidth,
        range
    ) as StatDensityGroupBy<T>
}

// todo other selecting options
fun <T, G> GroupBy<T, G>.statDensity(
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    range: Pair<Number, Number>? = null,
    selector: StatXNumericSelector<G>.() -> Unit,
): StatDensityGroupBy<T> {
    val appliedSelector = StatXNumericSelector(groups.concat()).apply(selector)
    return statDensity(
        appliedSelector.x,
        appliedSelector.weights,
        n,
        trim,
        adjust,
        kernel,
        fullScanMax,
        bandWidth,
        range
    )
}


fun <T, G> GroupBy<T, G>.statDensity(
    x: ColumnAccessor<Number>,
    weights: ColumnAccessor<Number>? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    range: Pair<Number, Number>? = null,
): StatDensityGroupBy<T> {
    val commonRange = range ?: groups.concat()[x].values().map {
        it.toDouble()
    }.let { it.min() to it.max() }
    return StatDensityGroupBy(updateGroups {
        statDensity(x, weights, n, trim, adjust, kernel, fullScanMax, bandWidth, commonRange)
    })
}

fun <T, G> GroupBy<T, G>.statDensity(
    x: String,
    weights: String? = null,
    n: Int = 512,
    trim: Boolean = false,
    adjust: Double = 1.0,
    kernel: Kernel = Kernel.GAUSSIAN,
    fullScanMax: Int = 5000,
    bandWidth: BandWidth = BandWidth.Method.NRD0,
    range: Pair<Number, Number>? = null,
): StatDensityGroupBy<T> {
    return statDensity(
        column(x),
        weights?.let { column(it) },
        n, trim, adjust, kernel, fullScanMax, bandWidth, range
    )
}
