package org.jetbrains.kotlinx.statistics.plotting.smooth

import org.jetbrains.kotlinx.dataframe.DataColumn
import org.jetbrains.kotlinx.dataframe.api.*
import org.jetbrains.kotlinx.dataframe.columns.ColumnAccessor
import org.jetbrains.kotlinx.kandy.dsl.internal.*
import org.jetbrains.kotlinx.statistics.selectors.*
import org.jetbrains.kotlinx.statistics.plotting.smooth.models.StatSmoothGroupBy
import org.jetbrains.kotlinx.kandy.dsl.internal.dataframe.*

@Suppress("INVISIBLE_MEMBER")
fun <T, G> GroupBy<T, G>.statSmooth(
    x: DataColumn<Number>,
    y: DataColumn<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100
): StatSmoothGroupBy<T> {
    val keyNames = keys.columnNames()
    val newGroupBy =
        (concatFixed().getColumns(*keyNames.toTypedArray()) + x + y).toDataFrame().groupBy(*keyNames.toTypedArray())
    return newGroupBy.statSmooth(
        x.toColumnAccessor(),
        y.toColumnAccessor(),
        method, smootherPointCount
    ) as StatSmoothGroupBy<T>
}

// todo other selecting options
fun <T, G> GroupBy<T, G>.statSmooth(
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
    selector: StatXNumYNumSelectorDSL<G>,
): StatSmoothGroupBy<T> {
    val appliedSelector = StatXNumYNumSelector(groups.concat()).apply(selector)
    return statSmooth(
        appliedSelector.x,
        appliedSelector.y,
        method, smootherPointCount
    )
}


fun <T, G> GroupBy<T, G>.statSmooth(
    x: ColumnAccessor<Number>,
    y: ColumnAccessor<Number>,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
): StatSmoothGroupBy<T> {
    return StatSmoothGroupBy(updateGroups {
        statSmooth(x, y, method, smootherPointCount)
    })
}

fun <T, G> GroupBy<T, G>.statSmooth(
    x: String,
    y: String,
    method: SmoothMethod = SmoothMethod.LOESS(),
    smootherPointCount: Int = 100,
): StatSmoothGroupBy<T> {
    return statSmooth(
        column(x),
        column(y),
        method, smootherPointCount
    )
}
